import logging as log
import os

import openai

from ai_chat.chat import Chat
from ai_chat.types import AIFunctions
from ai_chat import Function



class OpenaiChat(Chat):
    def chat_complete(self, prompt, functions: AIFunctions) -> tuple[str, str, Function | None]:
        openai.api_key = os.getenv("OPENAI_API_KEY")

        args = dict(
            messages=prompt,
            **self.ai.model_params,
            n=1,
        )

        if functions:
            args['functions'] = functions.openai_dict()

        log.debug(args)

        result = openai.ChatCompletion.create(**args)

        log.info("chat complete: %s", result)

        role = "assistant"
        message = result.choices[0].message
        content = message.content

        func = None
        if getattr(message, "function_call", None):
            function = message.function_call
            role = "function_call"
            func = Function(name=function.pop('name'), arguments=function.pop('arguments'), **function)

        return role, content, func



import contextlib
import os
import time

def uuid():
    return os.urandom(16).hex()


@contextlib.contextmanager
def assert_wait_for(func, secs=5):
    t0 = time.monotonic()
    while True:
        try:
            assert func()
            return
        except Exception as e:
            ex = e
        if time.monotonic() > (t0 + secs):
            raise ex


def summarize_json(data, list_limit=5, key_limit=5):
    "Recursively reduce data to just the first X nested keys and list items"
    if not isinstance(data, (list, dict)):
        return data
    if isinstance(data, list):
        return [summarize_json(item, list_limit, key_limit) for item in data[:list_limit]]
    if isinstance(data, dict):
        all_keys = list(data.keys())
        kept_keys = all_keys[:key_limit]
        truncated_keys = all_keys[key_limit:]
        d = dict([
            (key, summarize_json(data[key], list_limit, key_limit))
            for key in kept_keys
        ])
        if truncated_keys:
            d["_truncated_keys"] = truncated_keys
        return d



import dataclasses

from ai_chat.defaults import DEFAULT_CHAT_MODEL, DEFAULT_TEMPERATURE, DEFAULT_MAX_TOKENS, DEFAULT_ERROR_PREFIX

from ai_functions import AIFunctions


class AiConfig:
    """Persona and configuration"""
    id: str
    system: str
    error_prefix: str

    def __init__(self, *, id, system, error_prefix=DEFAULT_ERROR_PREFIX, functions: AIFunctions = None,
                 model_params=None, **data):
        self.__dict__ = data
        self.id = id
        self.system = system
        self.error_prefix = error_prefix
        self.functions = functions
        self.model_params = {
            **dict(model=DEFAULT_CHAT_MODEL, temperature=DEFAULT_TEMPERATURE, max_tokens=DEFAULT_MAX_TOKENS),
            **(model_params or {})}


class Message:
    id: str
    role: str
    content: str
    thread_id: str

    def __init__(self, *, id, role, content, thread_id, **data):
        self.__dict__ = data
        self.id = id
        self.role = role
        self.content = content
        self.thread_id = thread_id


class Function:
    name: str
    arguments: str

    def __init__(self, *, name, arguments, **data):
        self.__dict__ = data
        self.name = name
        self.arguments = arguments


@dataclasses.dataclass
class ChatResponse:
    request_id: str
    response_id: str
    content: str
